<?php

namespace FiloBlu\Refilo\Model\Consumer;

use FiloBlu\Core\Framework\Model\Mail\MailerInterface;
use FiloBlu\Esb\Api\Data\MessageInterface;
use FiloBlu\Esb\Api\Data\MessageInterfaceFactory;
use FiloBlu\Esb\Api\Data\ObjectTypeDescriptorInterface;
use FiloBlu\Esb\Api\Data\StatusInterface;
use FiloBlu\Esb\Api\Data\StatusInterfaceFactory;
use FiloBlu\Esb\Core\Expression\EvaluatorInterfaceFactory;
use FiloBlu\Esb\Core\Expression\SwitchExpressionEvaluatorFactory;
use FiloBlu\Esb\Core\Expression\VariableExpanderInterface;
use FiloBlu\Esb\Core\Extractor\ObjectTypeFromMessage;
use FiloBlu\Esb\Framework\Consumer\AbstractConsumer;
use FiloBlu\Esb\Framework\Template\TemplateInterface;
use FiloBlu\Esb\Framework\Template\TemplateInterfaceFactory;
use Psr\Log\LoggerInterface;

/**
 * {
 *   "mail": {
 *          "from" : "from@example.com",
 *          "to" : [ "to1@example.com", "to2@example.com" ],
 *          "subject" :
 *          "template" : "",
 *          "template_variables" : [],
 *          "template_options" : []
 *   }
 * }
 *
 * Class ContactConsumer
 * @package FiloBlu\Refilo\Model\Consumer
 */
class ContactConsumer extends AbstractConsumer
{
    /** @var string  */
    const MAIL = 'mail';

    /** @var string */
    const VARIABLES = 'variables';

    /**
     * @var MailerInterface
     */
    private $mailer;
    /**
     * @var \FiloBlu\Core\Framework\Model\Mail\MessageInterfaceFactory
     */
    private $mailMessageFactory;
    /**
     * @var SwitchExpressionEvaluatorFactory
     */
    private $switchEvaluatorFactory;
    /**
     * @var TemplateInterfaceFactory
     */
    private $templateFactory;
    /**
     * @var VariableExpanderInterface
     */
    private $variableExpander;

    /**
     * ContactConsumer constructor.
     * @param EvaluatorInterfaceFactory $evaluatorFactory
     * @param ObjectTypeDescriptorInterface $objectTypeDescriptor
     * @param MessageInterfaceFactory $messageFactory
     * @param ObjectTypeFromMessage $objectTypeFromMessage
     * @param StatusInterfaceFactory $statusFactory
     * @param TemplateInterfaceFactory $templateFactory
     * @param SwitchExpressionEvaluatorFactory $switchEvaluatorFactory
     * @param MailerInterface $mailer
     * @param VariableExpanderInterface $variableExpander
     * @param \FiloBlu\Core\Framework\Model\Mail\MessageInterfaceFactory $mailMessageFactory
     * @param LoggerInterface $logger
     */
    public function __construct(
        EvaluatorInterfaceFactory $evaluatorFactory,
        ObjectTypeDescriptorInterface $objectTypeDescriptor,
        MessageInterfaceFactory $messageFactory,
        ObjectTypeFromMessage $objectTypeFromMessage,
        StatusInterfaceFactory $statusFactory,
        TemplateInterfaceFactory $templateFactory,
        SwitchExpressionEvaluatorFactory $switchEvaluatorFactory,
        MailerInterface $mailer,
        VariableExpanderInterface $variableExpander,
        \FiloBlu\Core\Framework\Model\Mail\MessageInterfaceFactory $mailMessageFactory,
        LoggerInterface $logger
    ) {
        parent::__construct($evaluatorFactory, $objectTypeDescriptor, $messageFactory, $objectTypeFromMessage, $statusFactory, $logger);
        $this->mailer = $mailer;
        $this->mailMessageFactory = $mailMessageFactory;
        $this->evaluatorFactory = $evaluatorFactory;
        $this->switchEvaluatorFactory = $switchEvaluatorFactory;
        $this->templateFactory = $templateFactory;
        $this->variableExpander = $variableExpander;
    }

    /**
     * @param MessageInterface $message
     * @return MessageInterface
     */
    public function consume(MessageInterface $message): MessageInterface
    {
        if (!$this->isConditionSatisfied($message->getPayload())) {
            $this->getStatus()
                ->setCode(StatusInterface::SKIPPED)
                ->setOutputData('Condition was not satisfied');
            return $this->messageFactory->create();
        }

        $this->mailer->send($this->getMailMessage($message));

        $this->getStatus()->setCode(StatusInterface::SUCCESS);
        return $this->messageFactory->create();
    }

    /**
     * @param MessageInterface $message
     * @return mixed
     */
    public function getMailMessage(MessageInterface $message)
    {
        $payload = $message->getPayload();
        $emailMessage = $this->mailMessageFactory->create();

        $parameters = $this->getConfiguration()->getParameters();

        $templateVariables = $parameters->getDataByPath(self::VARIABLES) ?? [];
        $templateBody = $parameters->getDataByPath(self::MAIL) ?? [];

        $switchEvaluator = $this->switchEvaluatorFactory->create();

        foreach ($templateVariables as $name => $value) {
            $name = $this->variableExpander->expand($name, $payload);

            if ($switchEvaluator->is($value)) {
                $templateVariables[$name] = $switchEvaluator->evaluate($value, $payload);
                continue;
            }

            $templateVariables[$name] = $this->variableExpander->expand($value, $payload);
        }

        /** @var TemplateInterface $template */
        $template = $this->templateFactory->create();

        $mailRendered = $template
            ->setVariables($templateVariables)
            ->setTemplate($templateBody)
            ->render();

        $emailMessage
            ->setFrom($mailRendered->getDataByPath('from') ?? [])
            ->setTo($mailRendered->getDataByPath('to') ?? [])
            ->setTemplate($mailRendered->getDataByPath('template'))
            ->setTemplateOptions($mailRendered->getDataByPath('template_options') ?? [])
            ->setTemplateVariables($mailRendered->getDataByPath('template_variables') ?? []);

        $replyTo = $mailRendered->getDataByPath('reply_to');

        if ($replyTo) {
            $emailMessage->setReplyTo($replyTo);
        }

        return $emailMessage;
    }

    public function getName(): string
    {
        return 'Refilo Form Contact Consumer';
    }

    public function getUniqueName(): string
    {
        return 'FiloBlu_Refilo::form_contact_consumer';
    }
}
